{% extends "admin/change_form.html" %}

{% block object-tools-items %}
  {% for tool in objectactions %}
    <li class="objectaction-item" data-tool-name="{{ tool.name }}">
      {% url tools_view_name pk=object_id tool=tool.name as action_url %}
      {% include 'django_object_actions/action_trigger.html' %}
    </li>
  {% endfor %}
  {{ block.super }}
{% endblock %}
